module Matching (Matcher(..),
		 referencematcher,fastmatcher,
		 forceonestep,comparematchers,checkmatcher,tracematcher,
		 betareduce)
where
import qualified ReferenceMatching as R
import qualified FastMatching as F
import List
import PPrint
import Substitutions
import Expressions
import Names
import IO
import IOExts
import Monad


data Matcher = 
    Matcher {
	     simplematch   :: Exp -> Exp -> [Subst],
	     simplematch'  :: Subst -> Closure -> Closure -> [Subst],
	     onestepmatch  :: Exp -> Closure -> Name [Subst],
	     onestepmatch' :: Subst -> Closure -> Closure -> Name [Subst],
	     twostepmatch  :: Exp -> Closure -> Name [Subst],
	     twostepmatch' :: Subst -> Closure -> Closure -> Name [Subst],
	     viable       :: Exp -> Closure -> Bool
	    }

doreduce :: Matcher -> Matcher

doreduce m = 
    Matcher {
	     simplematch = \p t -> simplematch m (red p) (red t)
	    ,simplematch' = \s (p,bs) (t,cs) 
	                      -> simplematch' m s (red p,bs) (red t,cs)
	    ,onestepmatch = \p (t,bs)
	                      -> onestepmatch m (red p) (red t,bs)
            ,onestepmatch' = \s (p,bs) (t,cs)
                              -> onestepmatch' m s (red p,bs) (red t,cs)
            ,twostepmatch = \p (t,bs)
                              -> twostepmatch m (red p) (red t,bs)
            ,twostepmatch' = \s (p,bs) (t,cs)
	                      -> twostepmatch' m s (red p,bs) (red t,cs)
	    ,viable = viable m
	    }
  where red = etaContract . betareduce 

referencematcher = doreduce $
      Matcher {
	       simplematch = \ p t -> map R.anticonvertSubst $
	                          R.simplematch (R.convert p) 
                                                (R.convert t)
	      ,simplematch' = \s (p,bs) (t,cs) -> map R.anticonvertSubst $
	                          R.simplematch' (R.convertSubst s)
	                                         (R.convert' (map fst bs) p,bs)
                                                 (R.convert' (map fst cs) t,cs)
	      ,onestepmatch = \p (t,bs) -> liftM (map R.anticonvertSubst) $
	                          R.onestepmatch (R.convert p)
                                                 (R.convert' (map fst bs) t,bs)
	      ,onestepmatch' = \s (p,bs) (t,cs) -> 
	                         liftM (map R.anticonvertSubst) $
	                          R.onestepmatch' (R.convertSubst s)
                                                 (R.convert' (map fst bs) p,bs)
                                                 (R.convert' (map fst cs) t,cs)
	      ,twostepmatch = \p (t,bs) -> liftM (map R.anticonvertSubst) $
	                          R.twostepmatch (R.convert p)
                                                (R.convert' (map fst bs) t,bs)
	      ,twostepmatch' = \s (p,bs) (t,cs) -> 
	                         liftM (map R.anticonvertSubst) $
	                          R.twostepmatch' (R.convertSubst s)
                                                 (R.convert' (map fst bs) p,bs)
                                                 (R.convert' (map fst cs) t,cs)
	      ,viable = \p t -> True
	      }


fastmatcher = doreduce $
      Matcher {
	       simplematch = \ p t -> map F.anticonvertSubst $
	                          F.simplematch (F.convert p) 
                                                (F.convert t)
	      ,simplematch' = \s (p,bs) (t,cs) -> map F.anticonvertSubst $
	                          F.simplematch' (F.convertSubst s)
	                                         (F.convert' (map fst bs) p,bs)
                                                 (F.convert' (map fst cs) t,cs)
	      ,onestepmatch = \p (t,bs) -> liftM (map F.anticonvertSubst) $
	                          F.onestepmatch (F.convert p)
                                                 (F.convert' (map fst bs) t,bs)
	      ,onestepmatch' = \s (p,bs) (t,cs) -> 
	                         liftM (map F.anticonvertSubst) $
	                          F.onestepmatch' (F.convertSubst s)
                                                 (F.convert' (map fst bs) p,bs)
                                                 (F.convert' (map fst cs) t,cs)
	      ,twostepmatch = \p (t,bs) -> liftM (map F.anticonvertSubst) $
	                          F.twostepmatch (F.convert p)
                                                (F.convert' (map fst bs) t,bs)
	      ,twostepmatch' = \s (p,bs) (t,cs) -> 
	                         liftM (map F.anticonvertSubst) $
	                          F.twostepmatch' (F.convertSubst s)
                                                 (F.convert' (map fst bs) p,bs)
                                                 (F.convert' (map fst cs) t,cs)
	      ,viable = \p (t,bs) -> 
                        F.viable (F.convert p) (F.convert' (map fst bs) t)
	      }


forceonestep a = a { twostepmatch = onestepmatch a
		   , twostepmatch' = onestepmatch' a}

checkmatcher m = 
    Matcher {
	     simplematch = \p t -> map (checkm p t) (simplematch m p t)
	    ,simplematch' = \s (p,bs) (t,cs) 
                 -> map (checkm p t) (simplematch' m s (p,bs) (t,cs))
	    ,onestepmatch = \p (t,bs)
	         -> liftM (map (checkm p t)) (onestepmatch m p (t,bs))
	    ,onestepmatch' = \s (p,bs) (t,cs)
                 -> liftM (map (checkm p t)) (onestepmatch' m s (p,bs) (t,cs))
	    ,twostepmatch = \p (t,bs)
	         -> liftM (map (checkm p t)) (twostepmatch m p (t,bs))
	    ,twostepmatch' = \s (p,bs) (t,cs)
                 -> liftM (map (checkm p t)) (twostepmatch' m s (p,bs) (t,cs))
	    ,viable = viable m
	    }

checkm :: Exp -> Exp -> Subst -> Subst
checkm p t s = if p'' == t
	       then s
	       else errtrace dumpinfo s
  where dumpinfo = "\n\nMismatch!\nWhile matching " ++ prettyExp p
                     ++ " against " ++ prettyExp t
                     ++ "\nBad match:\n" ++ prettySubst s
                     ++ "\nsubstituted pattern: " ++ prettyExp p'
                     ++ "\nbetareduced: " ++ prettyExp p''
                     ++ "\n[Raw terms: pattern = " ++ show p ++ "\n"
                     ++ "\n term = " ++ show t ++ "\n"
                     ++ "\n subst = " ++ show (fst s) ++ "\n"
                     ++ "\n substpat = " ++ show p' ++ "\n"
                     ++ "\n betasubstpat = " ++ show p'' ++ "\n"
                     ++ "\n(End of error dump)\n\n"
        p' = appSubst s p
        p'' = betareduce p'


tracematcher m 
    = Matcher {
	       simplematch = \p t -> tracem p t (simplematch m p t)
	      ,simplematch' = \s (p,bs) (t,cs) 
                 -> tracem p t (simplematch' m s (p,bs) (t,cs))
	      ,onestepmatch = \p (t,bs)
	         -> liftM (tracem p t) (onestepmatch m p (t,bs))
	      ,onestepmatch' = \s (p,bs) (t,cs)
                 -> liftM (tracem p t) (onestepmatch' m s (p,bs) (t,cs))
	      ,twostepmatch = \p (t,bs)
	         -> liftM (tracem p t) (twostepmatch m p (t,bs))
	      ,twostepmatch' = \s (p,bs) (t,cs)
                 -> liftM (tracem p t) (twostepmatch' m s (p,bs) (t,cs))
	      ,viable = \p (t,bs) -> (viable m p (t,bs))
	      }

tracem p t = errtrace 
	      ("matching "++prettyExp p++" against "++prettyExp t++"\n")
              
tracev p t res = errtrace ("viability check - "++prettyExp p++" against "
                           ++prettyExp t++" - " 
			   ++if res then "viable" else "not viable" ++  " \n")
		 res

comparematchers a b 
    = Matcher {simplematch = \p t -> comparem p t 
	                                (simplematch a p t)
		                        (simplematch b p t)

              ,simplematch' = \s p'@(p,bs) t'@(t,cs) -> comparem p t
                                                      (simplematch' a s p' t')
                                                      (simplematch' b s p' t')

              ,onestepmatch = \p (t,bs) ->
                     do query <- onestepmatch a p (t,bs)
                        correct <- onestepmatch b p (t,bs)
                        return $ comparem p t query correct

              ,onestepmatch' = \s (p,bs) (t,cs) ->
                     do query <- onestepmatch' a s (p,bs) (t,cs)
                        correct <- onestepmatch' b s (p,bs) (t,cs)
                        return $ comparem p t query correct

              ,twostepmatch = \p (t,bs) ->
                     do query <- twostepmatch a p (t,bs)
                        correct <- twostepmatch b p (t,bs)
                        return $ comparem p t query correct

              ,twostepmatch' = \s (p,bs) (t,cs) ->
                     do query <- twostepmatch' a s (p,bs) (t,cs)
                        correct <- twostepmatch' b s (p,bs) (t,cs)
                        return $ comparem p t query correct

       	      ,viable = viable b
	      }

errtrace str exp = unsafePerformIO (
 				    do hPutStr stderr str
                                       return exp
                                   )

comparem p t a b = if length a /= length b 
                   then errtrace dumpinfo b 
                   else b
  where dumpinfo = "\n\nMismatch!\nWhile matching " ++ prettyExp p
                     ++ " against " ++ prettyExp t
                     ++ "\nFirst gives results:\n" ++ prettySs a 
                     ++ "\nSecond gives results:\n" ++ prettySs b
                     ++ "\n(End of error dump)\n\n"


betareduce :: Exp -> Exp
betareduce e@(Var _ _) = e
betareduce e@(Con _ _) = e
betareduce (Lam p e t)  = Lam p (betareduce e) t 
betareduce (Ap e1 e2 t) 
   = case e1' of
         (Lam p b u) -> if null matches then Ap e1' e2' t 
                        else betareduce (appSubst match b)
                            where matches = simplematch' fastmatcher 
                                                         idSubst 
                                                         (p,[])
                                                         (e2',tfreevars e2')
                                  match = head matches
         _           -> Ap e1' e2' t
       where e1' = betareduce e1
             e2' = betareduce e2


step :: Exp -> Exp

step e@(Var _ _) = e
step e@(Con _ _) = e
step e@(Lam x b t) = Lam x (step b) t
step e@(Ap e1 e2 t) 
        = case e1' of
             Lam (Var x u) b _ -> appSubst (head $ singleSubst x u e2') b
             _ -> Ap e1' e2' t
         where e1' = step e1
               e2' = step e2

twostep :: Exp -> Exp

twostep e@(Var _ _) = e
twostep e@(Con _ _) = e
twostep e@(Lam x b t) = Lam x (twostep b) t
twostep e@(Ap e1 e2 t) 
        = case e1' of
            Lam (Var x u) b _ -> step (appSubst (head $ singleSubst x u e2') b)
            _ -> Ap e1' e2' t
         where e1' = twostep e1
               e2' = twostep e2
