// MouseUpdater.h: interface for the CMouseUpdater class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MOUSEUPDATER_H__8D704A6D_41AD_4082_82AA_59D36967BEA8__INCLUDED_)
#define AFX_MOUSEUPDATER_H__8D704A6D_41AD_4082_82AA_59D36967BEA8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "PanDisp.h"
#include "Invalidator.h"
#include <vector>

//
// CLASS:   CMouseUpdater
//
// PURPOSE: The view window for a Pan effect lets you use
//          the mouse to provide input. The CMouseUpdater class
//          is used to relay the change in mouse position from
//          the view window to the Pan generated code.
//
class CMouseUpdater  
{
public:
    CMouseUpdater() { }
    virtual ~CMouseUpdater() { }

    void AddMouseListener(FloatUpdater* upd, void* pParamX,void* pParamY,
                          CInvalidator* pInvalidator)
    {
        MouseInfo mI;

        mI.updater = upd;
        mI.pParamX = pParamX;
        mI.pParamY = pParamY;
        mI.pInvalidator = pInvalidator;
        m_listeners.push_back(mI);
    }

    void UpdateMousePos(float newX, float newY)
    {
       std::vector<MouseInfo>::const_iterator it = m_listeners.begin();

       while (it != m_listeners.end()) {
          it->updater->UpdateFloat(it->pParamX, newX);
          it->updater->UpdateFloat(it->pParamY, newY);
          it->pInvalidator->Invalidate();
          it++;
       }
    }

private:
    typedef struct {
        FloatUpdater*     updater;
        void*             pParamX;
        void*             pParamY;
        CInvalidator* pInvalidator;
    } MouseInfo;

    std::vector<MouseInfo> m_listeners;
};

#endif // !defined(AFX_MOUSEUPDATER_H__8D704A6D_41AD_4082_82AA_59D36967BEA8__INCLUDED_)
