// NewDlg.h: interface and implementation of the CNewDlg class.
//
/////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_NEWDLG_H__193E6383_3193_40DB_B567_53DE7AAD2718__INCLUDED_)
#define AFX_NEWDLG_H__193E6383_3193_40DB_B567_53DE7AAD2718__INCLUDED_


#include "stdafx.h"
#include "resource.h"
#include <windowsx.h>
#include <vector>
#include <algorithm>
using namespace std;

// {E44A391E-8C72-4164-A82C-426E3541C40B}
static const GUID CATID_PanImage = 
{ 0xe44a391e, 0x8c72, 0x4164, { 0xa8, 0x2c, 0x42, 0x6e, 0x35, 0x41, 0xc4, 0xb } };

class CNewDlg : public CDialogImpl<CNewDlg>  
{
    typedef struct {
        LPTSTR   name;
        CLSID    clsid;
        LPTSTR   progID;
    } ComponentData;

    vector<ComponentData> clsidMap;
    CLSID*  m_clsid;
    bool*   m_bHandled;
    PTSTR*  m_progID;
    LPTSTR* m_effectName;

public:
	enum { IDD = IDD_NEWDOC };

private:
    // this could be sep'ed out. ToDo.
    static ComponentData mkComponentData(CLSID& clsid, LPCTSTR str, LPCTSTR progID) {

        ComponentData cData;
        LPTSTR pName = new TCHAR[_tcslen(str) + 1];
        _tcscpy(pName, str);

        LPTSTR pProgID = new TCHAR[_tcslen(progID) + 1];
        _tcscpy(pProgID, progID);

        memcpy(&cData.clsid,&clsid,sizeof(CLSID));
        cData.progID = pProgID;
        cData.name   = pName;

        return cData;
    }

    static bool lessCData(ComponentData c1, ComponentData c2) {  return (_tcsicmp(c1.name,c2.name) < 0); }

public:

	BEGIN_MSG_MAP(CNewDlg)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
        COMMAND_CODE_HANDLER(LBN_DBLCLK, OnCloseCmd)
		COMMAND_ID_HANDLER(IDOK, OnCloseCmd)
		COMMAND_ID_HANDLER(IDCANCEL, OnCloseCmd)
	END_MSG_MAP()

    CNewDlg() : m_clsid(NULL), m_bHandled(NULL), m_progID(NULL), m_effectName(NULL) { };
    CNewDlg(CLSID* c, PTSTR* pName, bool* bHandled, LPTSTR* pEffectName) 
        : m_clsid(c),
          m_bHandled(bHandled),
          m_progID(pName),
          m_effectName(pEffectName) { };
    virtual ~CNewDlg()
    { 
        ComponentData* pData;
        
        // free up the ComponentData entries
        while (clsidMap.size() > 0) {
            pData = &(ComponentData)clsidMap.front();
            delete [] pData->name;
            delete [] pData->progID;
            clsidMap.erase(clsidMap.begin());
       
        }
    };



	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		CenterWindow(GetParent());
        Populate(GetDlgItem(IDC_LISTBOX), clsidMap);
        sort(clsidMap.begin(),clsidMap.end(),lessCData);
		return TRUE;
	}

	LRESULT OnCloseCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
        bool hasSelection = false;
        HWND hWndLB = GetDlgItem(IDC_LISTBOX);
        if (wID != IDCANCEL) {

            int c = ListBox_GetCurSel(hWndLB);
            if (m_clsid && c != LB_ERR) {
                *m_clsid  = clsidMap[c].clsid;
                *m_progID = clsidMap[c].progID;

                // Fetch the name of the effect selected.
                if (m_effectName) {
                    *m_effectName = new TCHAR[ListBox_GetTextLen(hWndLB,c) + 1];
                    if (*m_effectName) {
                        ListBox_GetText(hWndLB,c,*m_effectName);
                    }
                }
                hasSelection = true;
            }
            wID = IDOK;
        }
        if (m_bHandled) {
            *m_bHandled = hasSelection;
        }
		EndDialog(wID);
		return 0;
	}

    static void Populate(HWND hWnd, vector<ComponentData>& clsidMap)
    {
        USES_CONVERSION;
        ICatInformation *pci = 0;

        // Fire up the Component Category Manager
        HRESULT hr = CoCreateInstance(
            CLSID_StdComponentCategoriesMgr, 0, 
            CLSCTX_ALL, IID_ICatInformation, (void**)&pci);


        if (SUCCEEDED(hr)) {
            // fish out the components that implement the PanImage category.
            IEnumCLSID *pec = 0;
            CATID rgcid;
            rgcid = CATID_PanImage;

            hr = pci->EnumClassesOfCategories(1, &rgcid, 0, 0, &pec);
            if (SUCCEEDED(hr)) {
                // walk list of CLSIDs 64 at a time
                enum { MAX = 64 };
                LPOLESTR clsidStr = NULL;
                TCHAR buf[MAX_PATH];
                TCHAR progID[MAX_PATH];
                HKEY hKey;
                CLSID rgclsid[MAX];
                hr = S_OK;
                DWORD dwType = REG_SZ;
                DWORD cbData = MAX_PATH - 1;
                LONG lr;
                do {
                    ULONG cActual = 0;
                    hr = pec->Next(MAX, rgclsid, &cActual);
                    if (SUCCEEDED(hr)) {
                        for (ULONG i = 0; i < cActual; i++) {
                            if (clsidStr) {
                                CoTaskMemFree(clsidStr);
                                clsidStr = NULL;
                            }
                            StringFromCLSID(rgclsid[i], &clsidStr);
                            _stprintf(buf, TEXT("CLSID\\%s"), OLE2T(clsidStr));
                            lr = RegOpenKeyEx(HKEY_CLASSES_ROOT,
                                         buf,
                                         0,
                                         KEY_READ,
                                         &hKey);
                            cbData = MAX_PATH - 1;
                            if ( lr == NO_ERROR ) {
                                lr = RegQueryValueEx(hKey, NULL, NULL, &dwType, (LPBYTE)buf, &cbData);
                                if ( lr == NO_ERROR ) {
                                    ListBox_AddString(hWnd, buf);
                                    HKEY hKeyProgID;

                                    progID[0] = TEXT('\0');

                                    lr = RegOpenKeyEx (hKey, TEXT("VersionIndependentProgID"), 0, KEY_READ, &hKeyProgID);
                                    if (lr == NO_ERROR ) {
                                        cbData = MAX_PATH - 1;
                                        lr = RegQueryValueEx(hKeyProgID, NULL, NULL, &dwType, (LPBYTE)progID, &cbData);
                                        RegCloseKey(hKeyProgID);
                                    }
                                    ComponentData cData = mkComponentData(rgclsid[i],buf,progID);
                                    clsidMap.push_back(cData);
                                }
                                RegCloseKey(hKey);
                            }
                        }
                    }
                } while (hr == S_OK);
                if (clsidStr) {
                   CoTaskMemFree(clsidStr);
                   clsidStr = NULL;
                }
                pec->Release();
            }
        }
        pci->Release();
        
    }

};

#endif // !defined(AFX_NEWDLG_H__193E6383_3193_40DB_B567_53DE7AAD2718__INCLUDED_)
